#include <avr/io.h>
#include "dcf77.h"

struct dcf77 dcf77;
struct dcf77 temp;

unsigned char dcf77_state = 0; 

struct {
	unsigned char parity:1;
	unsigned char pinstate:1;
	unsigned char error:1;
	unsigned char bit:1;
} state;

unsigned short interval;

void decode_bit(void)
{
	unsigned char bitvalues[] = {1, 2, 4, 8, 10, 20, 40, 80};
	if (!state.pinstate) {
		if ((interval >= 24) && (interval <= 28)) {
			state.bit=0;
		}
		else {
			if ((interval >= 50) && (interval <= 54)) {
				state.bit=1;
			}
			else
			{
				dcf77_state = 0;
				state.error=1;
			}
	}
		
		//Receive minute
		if ((temp.second >=21) && (temp.second <= 27)) {
			temp.minute+=bitvalues[temp.second - 21] * state.bit;
			state.parity^=state.bit;
		}
		
		//receive hour
		if ((temp.second >=29) && (temp.second <= 34)) {
			temp.hour+=bitvalues[temp.second - 29] * state.bit;
			state.parity^=state.bit;
		}
		
		//receive day
		if ((temp.second >=36) && (temp.second <= 41)) {
			temp.day+=bitvalues[temp.second - 36] * state.bit;
			state.parity^=state.bit;
		}
		
		//receive day of week
		if ((temp.second >=42) && (temp.second <= 44)) {
			temp.dayofweek+=bitvalues[temp.second - 42] * state.bit;
			state.parity^=state.bit;
		}
		
		//receive month
		if ((temp.second >=45) && (temp.second <= 49)) {
			temp.month+=bitvalues[temp.second - 45] * state.bit;
			state.parity^=state.bit;
		}
		
		//receive year
		if ((temp.second >=50) && (temp.second <= 57)) {
			temp.year+=bitvalues[temp.second - 50] * state.bit;
			state.parity^=state.bit;
		}
		

		if ((temp.second == 28) || (temp.second == 35) || (temp.second == 58)) {
			if (state.bit != state.parity)
			{
				state.error=1;
				dcf77_state = 0;
			}
			state.parity=0;
		}
		
		temp.second++;
	}
	else {
		if (interval > 450) {
			if ((temp.second == 59) && (!state.error)) {
				dcf77.hour=temp.hour;
				dcf77.minute=temp.minute;
				dcf77.second=0;
				dcf77.year=temp.year;
				dcf77.month=temp.month;
				dcf77.day=temp.day;
				dcf77.dayofweek=temp.dayofweek;
				dcf77_state = 1;
			}
			state.error=0;
			state.parity=0;
			temp.hour=0;
			temp.minute=0;
			temp.second=0;
			temp.day=0;
			temp.month=0;
			temp.year=0;
			temp.dayofweek=0;
		}
	}
}

void dcf77_trigger (void)
{
	static unsigned char prescaler;
	
	#if F_CPU % DEBOUNCE
  	OCR1A = F_CPU / DEBOUNCE - 1;		// compare DEBOUNCE - 1 times
	#endif
	
	if (((DCF77_PIN>>DCF77_PINX) & 0x01) != state.pinstate) {
		state.pinstate=~state.pinstate;
		decode_bit();
		interval=0;
	}
	else {
		interval++;
	}
  
  if (--prescaler == 0) {
    prescaler = (unsigned char) DEBOUNCE;
    if (++dcf77.second > 59) {
    	dcf77.second=0;
    	if (++dcf77.minute > 59) {
    		dcf77.minute=0;
    		if (++dcf77.hour > 23) {
    			dcf77.hour=0;
    		}
    	}
    }
	
	#if F_CPU % DEBOUNCE			// handle remainder
    OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1; // compare once per second
	#endif
  }
}
