#include <avr/io.h>
#include <avr/interrupt.h>
#include "fuehler.h"

unsigned char adr = 0x16;
unsigned char channel = 0;

static signed short temperatur[16];

volatile unsigned char i2c_status;
volatile unsigned short fuehler_counter=5<<8;

volatile unsigned char fuehler_busy;

signed short get_fuehler(unsigned char id)
{
	signed short temp;
	//while (fuehler_busy);
	fuehler_busy=1;
	temp=temperatur[id];
	fuehler_busy=0;
	return temp;
}

static void next_channel(void)
{
	channel++;
	if (channel > 0xF) {
		channel=0;
	}
	
	if (channel < 0x08) {
		adr=0x10 + 0x06;
	}
	else {
		adr=0x10 + 0x08;
	}
}

SIGNAL (SIG_2WIRE_SERIAL)
{
	static unsigned char lotemp;
	
	i2c_status=TWSR & 0xF8;
	
	switch (i2c_status) {
		case 0x08: //Start wurde gesendet
		case 0x10:
			TWDR=adr; //SLA+R/+W!
			TWCR=1<<TWINT | 1<<TWEN | 1<<TWIE;
			break;
			
			
		case 0x20: //SLA+W sent, no ack received
		case 0x48: //SLA+R sent, no Ack received
			next_channel();
			TWCR=1<<TWSTA | 1<<TWINT | 1<<TWEN | 1<<TWIE; //send Repeated Start Condition
			break;
			
		case 0x18: //SLA+W sent, ack received
			TWDR=channel & 0x07;
			TWCR=1<<TWINT | 1<<TWEN | 1<<TWIE; //Clear INT
			break;
		
		case 0x28: //Data byte transmitted, ack received
			adr++;
			//adr eins hochsetzten (+R)
			TWCR=1<<TWSTA | 1<<TWINT | 1<<TWEN | 1<<TWIE; //send Repeated Start Condition
			break;
			
		case 0x40: //SLA+R sent, ack received
			TWCR=1<<TWINT | 1<<TWEN | 1<<TWIE | 1<<TWEA; //Clear INT return ack
			break;
			
		case 0x50: //Data byte has been received, ACK has been returned
			lotemp=TWDR;
			TWCR=1<<TWINT | 1<<TWEN | 1<<TWIE;
			break;
			
		case 0x58: //Data byte has been received, NOT ACK has been returned
			//while (fuehler_busy);
			fuehler_busy=1;
			temperatur[channel]=lotemp | (TWDR << 8);
			fuehler_busy=0;
			next_channel();
			TWCR=1<<TWSTA | 1<<TWINT | 1<<TWEN | 1<<TWIE; //Clear INT, Send No ack, rep. Start!
			break;
			
		default:
			TWCR=0;
			return;
			//TWCR=1<<TWSTA | 1<<TWINT | 1<<TWEN | 1<<TWIE;
	}
	fuehler_counter=5<<8;
}

void fuehler_init(void)
{
	//SCL Clock 293 Hz
	TWBR=100; //TWI Bit rate
	TWSR=1<<TWPS1 | 1<<TWPS0; //Prescaler 64
	TWCR=1<<TWINT | 1<<TWSTA | 1<<TWEN | 1<<TWIE; //TWI Enable, Int lschen
	fuehler_counter=5*256;
}
