#include <avr/io.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include <math.h>
#include "heizung.h"
#include "fuehler.h"
#include "relais.h"
#include "dcf77.h"
#include "source.h"
#include "menu.h"
#include "holz.h"

#define CTRL_MEASURE	0
#define CTRL_ADJUST		1
#define CTRL_CLOSE		2
#define CTRL_NOP		3

unsigned char eheizexponent __attribute__ ((section (".eeprom"))) = 11;
unsigned char emaxvorlauf __attribute__ ((section (".eeprom"))) = 45;
signed char eminaussen __attribute__ ((section (".eeprom"))) = -12;
unsigned char eraumtemp[2] __attribute__ ((section (".eeprom"))) = {215, 195};
signed char eniveau __attribute__ ((section (".eeprom"))) = 0;
unsigned char eheiztimer[2] __attribute__ ((section (".eeprom"))) = {18, 88};
unsigned char eheizmode __attribute__ ((section (".eeprom"))) = 3;
unsigned char eheizgrenze[2] __attribute__ ((section (".eeprom"))) = {190, 120};

//Heizgrenze Stati
#define HGS_NO	0		//Heizgrenze(n) nicht erreicht
#define HGS_1		1		//Untere Heizgrenze berschritten
#define HGS_2		2		//Obere Heizgrenze berschritten

unsigned char heizgrenze_status;

unsigned short heizung_counter = 15<<8;
unsigned char controlstate = CTRL_CLOSE;

unsigned char heizung_status;
unsigned char heizung_modus;
signed short e_old;

signed long mixarc;

unsigned char get_mixer_percent(void)
{
	return mixarc * 100 / MAX_MIXARC;
}

signed short get_heizung_soll(void)
{
	float ti,	//Raumtemperatur
		ta,		//Auentemperatur
		tamin,	//minimale Auentemperatur
		tvmax,	//maximale Vorlauftemperatur
		tv,		//Vorlauf-Solltemperatur
		n,		//Heizkurvenexponent;
		niveau;	//Parallelverschiebung
	
	switch (heizung_modus)
	{
		case HEIZMODUS_VOLL:
			ti = eeprom_read_byte(&eraumtemp[0]) / 10.0;
			break;
		
		case HEIZMODUS_ABSENK:
			ti = eeprom_read_byte(&eraumtemp[1]) / 10.0;
			break;
		
		default:
			return get_fuehler(NTC_HEIZUNG_VORLAUF);
	}
		
//	ti = raumtemp / 10.0;
	ta = get_fuehler(NTC_AUSSEN) / 10.0;
	tvmax = eeprom_read_byte(&emaxvorlauf);
	n = eeprom_read_byte(&eheizexponent) / 10.0;
	niveau = eeprom_read_byte(&eniveau) / 10.0;
	tamin = (signed char) eeprom_read_byte(&eminaussen);
	
	if (ta < tamin)
		ta = tamin;
	
	if (ta >= ti)
	{
		tv = ti;
	}
	else
	{
		tv = ti+ ((tvmax-ti)* pow((ti-ta)/(ti-tamin), (1/n))) + niveau;
	}
	
	return (tv * 10);
}

static unsigned char heiz_timer(void)
{
	unsigned char thistime = dcf77.hour * 4 + dcf77.minute / 15;

	if (eeprom_read_byte(&eheiztimer[0]) <= eeprom_read_byte(&eheiztimer[1]))
		if ((thistime >= eeprom_read_byte(&eheiztimer[0])) && (thistime < eeprom_read_byte(&eheiztimer[1])))
			return 1;
		else
			return 0;
	else
		if ((thistime >= eeprom_read_byte(&eheiztimer[0])) || (thistime < eeprom_read_byte(&eheiztimer[1])))
			return 1;
		else
			return 0;
}

static void calc_heizmodus(void)
{
	switch (eeprom_read_byte(&eheizmode))
	{
		case EHEIZMODE_AUS:
			heizung_modus = HEIZMODUS_AUS;
			break;
		
		case EHEIZMODE_AUS_ABSENK:
			heizung_modus = HEIZMODUS_AUS ? HEIZMODUS_ABSENK : (heiz_timer());
			break;

		case EHEIZMODE_ABSENK_ABSENK:
			heizung_modus = HEIZMODUS_ABSENK;
			break;

		case EHEIZMODE_NORMAL_ABSENK:
			if (heiz_timer())
				heizung_modus = HEIZMODUS_VOLL;
			else
				heizung_modus = HEIZMODUS_ABSENK;
			break;

		case EHEIZMODE_NORMAL_NORMAL:
			heizung_modus = HEIZMODUS_VOLL;

		default:
			heizung_modus = HEIZUNG_AUS;
	}

	if (get_fuehler(NTC_AUSSEN) >= (eeprom_read_byte(&eheizgrenze[0]) + HEIZGRENZE_HYST))
		heizgrenze_status = HGS_2;
		
	if ((get_fuehler(NTC_AUSSEN) > (eeprom_read_byte(&eheizgrenze[1]) + HEIZGRENZE_HYST))
		 && (get_fuehler(NTC_AUSSEN) < eeprom_read_byte(&eheizgrenze[0]) - HEIZGRENZE_HYST))
		heizgrenze_status = HGS_1;
	
	if (get_fuehler(NTC_AUSSEN) < (eeprom_read_byte(&eheizgrenze[1]) - HEIZGRENZE_HYST))
		heizgrenze_status = HGS_NO;
	
	
	if ((heizung_modus == HEIZMODUS_VOLL) && (heizgrenze_status == HGS_2))
		heizung_modus = HEIZMODUS_AUS;
	
	if ((heizung_modus == HEIZMODUS_ABSENK) && (heizgrenze_status != HGS_NO))
		heizung_modus = HEIZMODUS_AUS;
}

void heizung_statetrigger(void)
{
	calc_heizmodus();
	
	if (heizung_modus == HEIZMODUS_AUS)
		heizung_status = HEIZUNG_AUS;
	else
		if (heizung_status == HEIZUNG_AUS)
			heizung_status = HEIZUNG_EIN;
	
	switch (heizung_status)
	{
		case HEIZUNG_EIN:
			if (get_sourcetype() == SOURCE_PUFFER)
			{
				if (get_sourcetemp() < PUFFER_MIN - 5)
					heizung_status = HEIZUNG_NOENERGY;
			}
			else
				if (get_sourcetemp() - 10 < get_heizung_soll())
					heizung_status = HEIZUNG_NOENERGY;
				
			break;
		
		case HEIZUNG_NOENERGY:
			if (get_sourcetemp() - 30 > get_heizung_soll())
				heizung_status = HEIZUNG_EIN;
			break;
	}
	
	if (heizung_status == HEIZUNG_EIN)
	{
		RELAIS_HEIZPUMPE_ON;
		if (controlstate >= CTRL_CLOSE)
		{
			controlstate = CTRL_MEASURE;
			heizung_ctrltrigger();
		}
	}
	else
	{
		RELAIS_HEIZPUMPE_OFF;
		if (controlstate < CTRL_CLOSE)
		{
			controlstate = CTRL_CLOSE;
			heizung_ctrltrigger();
		}
	}
}

void heizung_ctrltrigger(void)
{
	signed short e; //Regelabweichung
	unsigned short new_counter;
	
	switch (controlstate)
	{
		case CTRL_MEASURE:
			e=get_heizung_soll() - get_fuehler(NTC_HEIZUNG_VORLAUF);
			
			new_counter = abs((70 * e) + (60 * (e-e_old))); //Stellgre = Regelverstrkung * Regelabweichung
			
			if (new_counter > 128)
			{ 	
				if (e > 0)
				{
					mixarc+=new_counter;
					if (mixarc > MAX_MIXARC)
					{
						mixarc=MAX_MIXARC;
					}
					RELAIS_MIXER_OPEN;
				}
				else
				{
					mixarc-=new_counter;
					if (mixarc < 0)
					{
						mixarc=0;
					}
					RELAIS_MIXER_CLOSE;
				}
				controlstate = CTRL_ADJUST;
		 		RELAIS_MIXER_ON;
		 	}
		 	else
			{
		 		new_counter = 40<<8;
		 	}
		 	e_old=e;
			heizung_counter = new_counter;
		 		
			break;
			
		case CTRL_ADJUST:
			RELAIS_MIXER_OFF;
			RELAIS_MIXER_CLOSE;
			heizung_counter = 30<<8; //30s warten, dann messen und regeln...
			controlstate = CTRL_MEASURE;
			break;
			
		case CTRL_CLOSE:
			RELAIS_MIXER_CLOSE;
			RELAIS_MIXER_ON;
			controlstate = CTRL_NOP;
			heizung_counter = 120<<8;
			mixarc = 0;
			break;
			
		case CTRL_NOP:
			RELAIS_MIXER_OFF;
			heizung_counter = 30<<8;
			break;
			
	}
}
/*
	12345
1
2	*   *
3	 ***
4	*****
5	*****
6	 ***
7	*   *
8	
*/

/*
	12345
1	    
2	   **
3	 * *
4	*  *
5	*  *
6	 * * 
7	   **
8
*/
