typedef unsigned char byte;
typedef unsigned short word;

#include <avr/pgmspace.h>
#include "lcd_hd44780.h"

byte active_e = 7;

void pulse_e(void)
{
	unsigned short i;
	
	for (i=0;i<20;i++) {
		asm volatile("nop");
	}
	
	PORTD|=1<<active_e;
	
	for (i=0;i<20;i++) {
		asm volatile("nop");
	}
		
	PORTD&=~(1<<active_e);
	
	for (i=0;i<20;i++) {
		asm volatile("nop");
	}
}

void loop_while_busy(void)
{
	byte i;
	DDRC&=0xC3; //PC2-PC5 in
	PORTD=(PORTD & 0x0F) | (1<<PD4); //R/W setzen!

	PORTD|=1<<active_e; //E setzten
		for (i=0;i<30;i++) {
			asm volatile("nop");
		}
	
	loop_until_bit_is_clear(PINC, PC5);
	
	PORTD&=~(1<<active_e);
	for (i=0;i<30;i++) {
		asm volatile("nop");
	}
	
	PORTD&=~(1<<PD4); //RW lschen
	DDRC=(DDRC & 0xC3) | 0x3C; //PC2-PC5 out
}

void lcd_send_data(byte data)
{
	word i;
	loop_while_busy();
	PORTD|=1<<PD5; //RS setzten
	PORTC=(PORTC & 0xC3) | ((data >> 2) & 0x3C); //oberes nibble senden!
	//outp((inp(PINC) & 0xC3) | ((data >> 2) & 0x3C), PORTC); 
	pulse_e();
	PORTC=(PORTC & 0xC3) | ((data << 2) & 0x3C);
	//outp((inp(PINC) & 0xC3) | ((data << 2) & 0x3C), PORTC); //unteres nibble setzten
	pulse_e();
	for (i=0;i<255;i++) {
		asm volatile("nop");
	}
}

void lcd_send_comm(byte comm)
{
	byte i;
	loop_while_busy();
	PORTD&=~(1<<PD5); //RS lschen
	PORTC=(PORTC & 0xC3) | ((comm >> 2) & 0x3C); //oberes nibble senden!
	//outp((inp(PINC) & 0xC3) | ((comm >> 2) & 0x3C), PORTC);
	pulse_e();
	PORTC=(PORTC & 0xC3) | ((comm << 2) & 0x3C);
	//outp((inp(PINC) & 0xC3) | ((comm << 2) & 0x3C), PORTC);
	pulse_e();
	for (i=0;i<255;i++) {
		asm volatile("nop");
	}
}

void lcd_clear(void)
{
	lcd_send_comm(1);
}

void lcd_select(byte ctrl)
{
	if (ctrl == 2) {
		active_e=7;
	}
	else {
		active_e=6;
	}
}

void lcd_locate(byte pos)
{
	lcd_send_comm(0x80 | (pos & 0x7F));
}

void lcd_dsplcrs(byte flags)
{
	lcd_send_comm(0x08 + ((flags & 0x1C) >> 2)); //display on/off
}

void lcd_init(byte flags)
{
	word i;
	
	PORTC&=0xC3;
	PORTD&=0x0F;
	DDRC=(DDRC & 0xC3) | 0x3C;
	DDRD=(DDRD & 0x0F) | 0xF0;
	
	//Power On Delay
	for(i=0;i<65000;i++) {
		asm volatile("nop");
	}
		
	//Initialisieren	
	PORTC=(PINC & 0xC3) | 0xC;
	pulse_e();
	pulse_e();
	pulse_e();
	
	//4-bit Modus setzten
	PORTC=(PINC & 0xC3) | 0x08;
	pulse_e();
	
	lcd_send_comm(0x20 + ((flags & 0x03) << 2)); //function set
	lcd_dsplcrs(flags);
	lcd_send_comm(0x04 + ((flags & 0x60) >> 5)); //entry mode
	lcd_clear();
}

void lcd_puts(char *data)
{
	while(*data) {
		lcd_send_data(*data++);
	}
}

void lcd_puts_P(PGM_P s)
{
	register char c;
	
	while (c=pgm_read_byte(s++))
	{
		lcd_send_data(c);
	}
}
