#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "wintek2704.h"
#include "dcf77.h"
#include "menu.h"
#include "timebase.h"
#include "key.h"
#include "fuehler.h"
#include "solar.h"
#include "heizung.h"
#include "holz.h"
#include "source.h"
#include "uart.h"

#include "relais.h"


#include "menustr.h"

const char chars[] = {0x20, 0x11, 0x2E, 0x1F, 0x1F, 0x2E, 0x11, 0x20,
						0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28};

						
static void get_holz_ok(void)
{
	static unsigned long c = 100000;
	
	if (bit_is_clear(PINB, PB0))
	{
		c = 100000;
		RELAIS_HOLZPUMPE_ON
		if (holz_aktiv == 0)
		{
			holz_aktiv = 1;
			source_trigger();
		}
			
		holz_aktiv = 1;
	}
	else
		if (c)
		{
			if ((--c) == 0)
			{
				RELAIS_HOLZPUMPE_OFF;
				holz_aktiv = 0;
			}
		}
}

void init_io(void)
{
	unsigned char i;
	
	UBRRH=0;
	UBRRL=47; //47=9k6, 3=115k2 bei 7.3728 MHz Quarz
	UCSRB=1<<TXEN ;
	UCSRC=1<<URSEL | 1<<UCSZ1 | 1<<UCSZ0; //8N1
	
	//Display initialisierung
	lcd_init(LCD_DISP_ON);
	lcd_puts_P(str_startup);
	
	lcd_command(LCD_SET_CGRAM);
	for (i=0; i<16; i++)
	{
		lcd_write(pgm_read_byte(&chars[i]), 1);
	}
	lcd_gotoxy(0, 2);
	lcd_command(LCD_SET_CGRAM);
	for (i=0; i<16; i++)
	{
		lcd_write(pgm_read_byte(&chars[i]), 1);
	}
		
	
	//Interrupts aktivieren
	sei();
	
	//Gerte initialisieren	
	DDRA=0xFF; //Relais 1-8
	DDRC|=0xC0; //Relais 9-10
	DDRB=0;
	PORTB |= 1<<PB3; //PullUp fr Optokoppler/Holzkessel
	PORTB|=0xF1; //Pull Up fr Taster und Optokoppler
	fuehler_init();
	
	//warten, dass alle Temperaturen gemessen wurden...

	OCR1A=43200; //Nach 6s compare match!
	TCCR1B=1<<CS12 | 1<<CS10; //Prescaler 1024
	loop_until_bit_is_set(TIFR, OCF1A);
	TIFR|=1<<OCF1A;
	TCCR1B=0;
		
	//Fr Task-Scheduler
	OCR1A = F_CPU / DEBOUNCE - 1;
	TCCR1B = 1<<WGM12 | 1<<CS10; // divide by 1, clear on compare
	TIMSK = 1<<OCIE1A;

}

int main(void)
{
	init_io();
	
	unsigned char sec=0;
	
	unsigned char key;
	
	for (;;)
	{
		key=get_key();
		
		if (key)
		{
			menu_event(key);
		}
		
		if (sec != dcf77.second)
		{
			menu_event(EVENT_SECOND);
			sec=dcf77.second;
		}
		
		if ((timebase_flags & (1<<TBFLAG_SOLAR)))
		{
			timebase_flags &= ~(1<<TBFLAG_SOLAR);
			solar_trigger();
		}
		
		if ((timebase_flags & (1<<TBFLAG_HEIZUNG)))
		{
			timebase_flags &= ~(1<<TBFLAG_HEIZUNG);
			heizung_ctrltrigger();
		}
		
		if ((timebase_flags & (1<<TBFLAG_SOURCE)))
		{
			timebase_flags &= ~(1<<TBFLAG_SOURCE);
			source_trigger();
		}
		
		if ((timebase_flags & (1<<TBFLAG_UART)) && dcf77_state)
		{
			timebase_flags &= ~(1<<TBFLAG_UART);
			uart_trigger();
		}
		
		get_holz_ok();
	}
	
	return 0;
}
