#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdlib.h>
#include "menu.h"
#include "wintek2704.h"
#include "dcf77.h"
#include "fuehler.h"
#include "menustr.h"
#include "display.h"
#include "heizung.h"
#include "holz.h"
#include "oel.h"
#include "brauchwasser.h"
#include "source.h"

#include "timebase.h"

unsigned char menu_state = MS_STATUS;

static signed short currentval, minval, maxval;

static void display_heizstart(void);
static void display_heizstop(void);
static void display_maxvorlauf(void);
static void display_minaussen(void);
static void display_raumtemp2(void);
static void display_niveau(void);
static void display_oelstart(void);
static void display_oelstop(void);

struct menu_item_t
{
	unsigned char state;
	unsigned char event;
	void (*proc)(void);
	unsigned char nextstate;
};

void test_proc(void)
{
	holz_status = HOLZ_BRENNT;
	holz_trigger();
}

void test_proc2(void)
{

}



static void show_status(void)
{
	lcd_puts_P(str_status_menu);
	display_clock();
	lcd_gotoxy(15, 1);
	display_status((PGM_P*) str_brauchwasser_states, brauchwasser_status);
	lcd_gotoxy(10, 2);
	display_status((PGM_P*) str_heizung_states, heizung_status);
	lcd_gotoxy(9, 3);
	display_status((PGM_P*) str_source_types, get_sourcetype());
	lcd_putc(' ');
	display_dezival(get_sourcetemp(), 0xDF);
}

static void show_holzkessel(void)
{
	lcd_puts_P(str_holzkessel_menu);
	display_clock();
	lcd_gotoxy(12, 1);
	display_dezival(get_fuehler(NTC_HOLZ_VORLAUF), 0xDF);
	lcd_gotoxy(20, 1);
	display_dezival(get_fuehler(NTC_PUFFER_OBEN), 0xDF);
	lcd_gotoxy(12, 2);
	display_dezival(get_fuehler(NTC_HOLZ_RUECKLAUF), 0xDF);
	lcd_gotoxy(20, 2);
	display_dezival(get_fuehler(NTC_PUFFER_UNTEN), 0xDF);
	lcd_gotoxy(9, 3);
	display_status((PGM_P*) str_holz_states, holz_status);

	switch (holz_status)
	{
		case HOLZ_BRENNT:
			display_dezival(get_fuehler(NTC_HOLZ_VORLAUF) - get_fuehler(NTC_HOLZ_RUECKLAUF), 0xDF);
			break;
			
		case HOLZ_ABKUEHLEN:
			display_val(holz_delay, 0);
			break;
	}
}

static void show_brauchwasser(void)
{
	lcd_puts_P(str_brauchwasser_menu);
	display_clock();
	lcd_gotoxy(8, 1);
	display_dezival(get_fuehler(NTC_BOILER_OBEN), 0xDF);
	lcd_gotoxy(14, 1);
	display_status((PGM_P*) str_brauchwasser_states, brauchwasser_status);
	lcd_gotoxy(8, 2);
	display_dezival(get_fuehler(NTC_BOILER_UNTEN), 0xDF);
	lcd_gotoxy(8, 3);
	display_dezival(get_fuehler(NTC_SOLAR), 0xDF);
}

static void show_heizung(void)
{
	lcd_puts_P(str_heizung_menu);
	display_clock();
	
	lcd_gotoxy(12, 1);
	display_dezival(get_fuehler(NTC_HEIZUNG_VORLAUF), 0xDF);
	lcd_gotoxy(12, 2);
	display_dezival(get_fuehler(NTC_HEIZUNG_RUECKLAUF), 0xDF);
	lcd_gotoxy(19, 2),
	display_val(get_mixer_percent(), '%');
	lcd_gotoxy(7, 3);
	display_dezival(get_heizung_soll(), 0xDF);
	lcd_gotoxy(21, 3);
	display_dezival(get_fuehler(NTC_AUSSEN), 0xDF);
	lcd_gotoxy(19, 1);
	display_status((PGM_P*) str_heizung_states, heizung_status);
}

static void show_oelkessel(void)
{
	lcd_puts_P(str_oelkessel_menu);
	display_clock();
	lcd_gotoxy(12, 1);
	display_dezival(get_fuehler(NTC_OEL_VORLAUF), 0xDF);
	lcd_gotoxy(12, 2);
	display_dezival(get_fuehler(NTC_OEL_RUECKLAUF), 0xDF);
	lcd_gotoxy(9, 3);
	display_status((PGM_P*) str_oel_states, oel_status);
	
	lcd_gotoxy(22, 1);
	display_val(oel_laststart_h, ':');
	display_val(oel_laststart_m, 0);
}

/*----------------------------------------------------
									SETUP Menues
----------------------------------------------------*/
static void show_service(void)
{
	lcd_puts_P(str_service_menu);
}

static void show_oelmode(void)
{
	lcd_puts_P(str_oelmodus_menu);
	lcd_gotoxy(10, 1);
	lcd_command(LCD_DISP_ON);
	display_status((PGM_P*) str_oelmodes, eeprom_read_byte(&eoel_mode));
	lcd_gotoxy(6, 2);
	display_time(eeprom_read_byte(&eoel_timer[0]));
	lcd_gotoxy(6, 3);
	lcd_command(LCD_DISP_ON);
	display_time(eeprom_read_byte(&eoel_timer[1]));
}

static void show_heizmode(void)
{
	lcd_puts_P(str_heizmode_menu);
	lcd_gotoxy(10, 1);
	lcd_command(LCD_DISP_ON);
	display_status((PGM_P*) str_heizmodes, eeprom_read_byte(&eheizmode));
	lcd_gotoxy(6, 2);
	display_time(eeprom_read_byte(&eheiztimer[0]));
	lcd_gotoxy(6, 3);
	lcd_command(LCD_DISP_ON);
	display_time(eeprom_read_byte(&eheiztimer[1]));
}

static void show_heizkurve(void)
{
	lcd_puts_P(str_heizkurve_menu);
	lcd_gotoxy(15, 1);
	lcd_command(LCD_DISP_ON);
	display_dezival(eeprom_read_byte(&eheizexponent), 0);
	lcd_gotoxy(15, 2);
	display_val(eeprom_read_byte(&emaxvorlauf), 0xDF);
	lcd_gotoxy(15, 3);
	lcd_command(LCD_DISP_ON);
	display_val((signed char) eeprom_read_byte(&eminaussen), 0xDF);
}

static void show_raumniveau(void)
{
	lcd_puts_P(str_raumniveau_menu);
	lcd_gotoxy(14, 1);
	lcd_command(LCD_DISP_ON);
	display_dezival(eeprom_read_byte(&eraumtemp[0]), 0xDF);
	lcd_gotoxy(14, 2);
	display_dezival(eeprom_read_byte(&eraumtemp[1]), 0xDF);
	lcd_gotoxy(14, 3);
	lcd_command(LCD_DISP_ON);
	display_dezival((signed char) eeprom_read_byte(&eniveau), 0xDF);
}

static void show_heizgrenze(void)
{
	lcd_puts_P(str_heizgrenze_menu);
	lcd_gotoxy(14, 1);
	lcd_command(LCD_DISP_ON);
	display_dezival(eeprom_read_byte(&eheizgrenze[0]), 0);
	lcd_gotoxy(14, 2);
	display_dezival(eeprom_read_byte(&eheizgrenze[1]), 0);
}

static void show_bwprio(void)
{
	lcd_puts_P(str_bwprio_menu);
	lcd_gotoxy(2, 1);
	lcd_command(LCD_DISP_ON);
	display_val(eeprom_read_byte(&ebrauchwasserprio), 0xDF);
}

static void inc_val(void)
{
	if (++currentval > maxval)
		currentval = maxval;
}

static void dec_val(void)
{
	if (--currentval < minval)
		currentval = minval;
}

static void check_val(void)
{
	if ((currentval > maxval) || (currentval < minval))
		currentval = minval + (maxval - minval) / 2;
}

/*---------------------------------------------------
						Funktionen fr Servicemen
----------------------------------------------------*/
static void display_serviceval(void)
{
	show_service();
	lcd_gotoxy(1, 1);
	display_status((PGM_P*) str_services, currentval);
}

static void go_service(void)
{
	currentval = 6;
	minval = 0;
	maxval = 9;
	display_serviceval();
}

static void inc_service(void)
{
	inc_val();
	display_serviceval();
}

static void dec_service(void)
{
	dec_val();
	display_serviceval();
}

static void set_service(void)
{
	unsigned char state = 0;
	if (currentval < 8)
	{
		PORTA ^= 1<<currentval;
		state = PORTA & (1<<currentval);
	}
	else
	{
		PORTC ^= 1<<(currentval - 2);
		state = PORTC & (1<<(currentval - 2));
	}
	
	display_serviceval();
	if (state)
		lcd_puts_P(str_ein);
	else
		lcd_puts_P(str_aus);
}

/*----------------------------------------------------
								FUNKTIONEN FR lMODUS
----------------------------------------------------*/
static void display_oelmode(void)
{
	lcd_gotoxy(10, 1);
	display_status((PGM_P*) str_oelmodes, currentval);
}

static void go_oelmode(void)
{
	show_oelmode();
	currentval = eeprom_read_byte(&eoel_mode);
	minval = 0;
	maxval = 3;
	check_val();
	display_oelmode();
	lcd_command(LCD_DISP_ON_BLINK);
}

static void inc_oelmode(void)
{
	inc_val();
	display_oelmode();
}

static void dec_oelmode(void)
{
	dec_val();
	display_oelmode();
}

static void set_oelmode(void)
{
	eeprom_write_byte(&eoel_mode, currentval);
	show_oelmode();
	
	currentval = eeprom_read_byte(&eoel_timer[0]);
	minval = 0;
	maxval = 95;
	check_val();
	display_oelstart();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR LSTART
----------------------------------------------------*/
static void display_oelstart(void)
{
	lcd_gotoxy(6, 2);
	display_time(currentval);
}

static void inc_oelstart(void)
{
	inc_val();
	display_oelstart();
}

static void dec_oelstart(void)
{
	dec_val();
	display_oelstart();
}

static void set_oelstart(void)
{
	eeprom_write_byte(&eoel_timer[0], currentval);
	show_oelmode();
	
	currentval = eeprom_read_byte(&eoel_timer[1]);
	minval = 0;
	maxval = 95;
	check_val();
	display_oelstop();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR LSTOP
----------------------------------------------------*/
static void display_oelstop(void)
{
	lcd_gotoxy(6, 3);
	display_time(currentval);
}

static void inc_oelstop(void)
{
	inc_val();
	display_oelstop();
}

static void dec_oelstop(void)
{
	dec_val();
	display_oelstop();
}

static void set_oelstop(void)
{
	eeprom_write_byte(&eoel_timer[1], currentval);
	go_oelmode();
}


/*----------------------------------------------------
								FUNKTIONEN FR HEIZMODUS
----------------------------------------------------*/
static void display_heizmode(void)
{
	lcd_gotoxy(10, 1);
	display_status((PGM_P*) str_heizmodes, currentval);
}

static void go_heizmode(void)
{
	show_heizmode();
	currentval = eeprom_read_byte(&eheizmode);
	minval = 0;
	maxval = 4;
	check_val();
	display_heizmode();
	lcd_command(LCD_DISP_ON_BLINK);
}

static void inc_heizmode(void)
{
	inc_val();
	display_heizmode();
}

static void dec_heizmode(void)
{
	dec_val();
	display_heizmode();
}

static void set_heizmode(void)
{
	eeprom_write_byte(&eheizmode, currentval);
	show_heizmode();
	
	currentval = eeprom_read_byte(&eheiztimer[0]);
	minval = 0;
	maxval = 95;
	check_val();
	display_heizstart();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR HEIZSTART
----------------------------------------------------*/
static void display_heizstart(void)
{
	lcd_gotoxy(6, 2);
	display_time(currentval);

}

static void inc_heizstart(void)
{
	inc_val();
	display_heizstart();
}

static void dec_heizstart(void)
{
	dec_val();
	display_heizstart();
}

static void set_heizstart(void)
{
	eeprom_write_byte(&eheiztimer[0], currentval);
	show_heizmode();
	
	currentval = eeprom_read_byte(&eheiztimer[1]);
	
	check_val();
	display_heizstop();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR HEIZSTOP
----------------------------------------------------*/
static void display_heizstop(void)
{
	lcd_gotoxy(6, 3);
	display_time(currentval);
}

static void inc_heizstop(void)
{
	inc_val();
	display_heizstop();
}

static void dec_heizstop(void)
{
	dec_val();
	display_heizstop();
}

static void set_heizstop(void)
{
	eeprom_write_byte(&eheiztimer[1], currentval);
	go_heizmode();
}


/*----------------------------------------------------
								FUNKTIONEN FR HEIZEXPONENT
----------------------------------------------------*/
static void display_heizexponent(void)
{
	lcd_gotoxy(15, 1);
	display_dezival(currentval, 0);
}

static void go_heizexponent(void)
{
	show_heizkurve();
	currentval = eeprom_read_byte(&eheizexponent);
	minval = 1;
	maxval = 30;
	check_val();
	display_heizexponent();
	lcd_command(LCD_DISP_ON_BLINK);
}

static void inc_heizexponent(void)
{
	inc_val();
	display_heizexponent();
}

static void dec_heizexponent(void)
{
	dec_val();
	display_heizexponent();
}

static void set_heizexponent(void)
{
	eeprom_write_byte(&eheizexponent, currentval);
	show_heizkurve();
	
	currentval = eeprom_read_byte(&emaxvorlauf);
	minval = 35;
	maxval = 60;
	check_val();
	display_maxvorlauf();
	lcd_command(LCD_DISP_ON_BLINK);
}


/*----------------------------------------------------
								FUNKTIONEN FR MAXVORLAUF
----------------------------------------------------*/
static void display_maxvorlauf(void)
{
	lcd_gotoxy(15, 2);
	display_val(currentval, 0xDF);
}

static void inc_maxvorlauf(void)
{
	inc_val();
	display_maxvorlauf();
}

static void dec_maxvorlauf(void)
{
	dec_val();
	display_maxvorlauf();
}

static void set_maxvorlauf(void)
{
	eeprom_write_byte(&emaxvorlauf, currentval);
	show_heizkurve();
	
	currentval = (signed char) eeprom_read_byte(&eminaussen);
	minval = -15;
	maxval = 0;
	check_val();
	display_minaussen();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR MINAUSSEN
----------------------------------------------------*/
static void display_minaussen(void)
{
	lcd_gotoxy(15, 3);
	display_val(currentval, 0xDF);
}

static void inc_minaussen(void)
{
	inc_val();
	display_minaussen();
}

static void dec_minaussen(void)
{
	dec_val();
	display_minaussen();
}

static void set_minaussen(void)
{
	eeprom_write_byte(&eminaussen, currentval);
	go_heizexponent();
}


/*----------------------------------------------------
								FUNKTIONEN FR RAUMTEMP1
----------------------------------------------------*/
static void display_raumtemp1(void)
{
	lcd_gotoxy(14, 1);
	display_dezival(currentval, 0);
}

static void go_raumtemp1(void)
{
	show_raumniveau();
	currentval = eeprom_read_byte(&eraumtemp[0]);
	minval = 160;
	maxval = 255;
	check_val();
	display_raumtemp1();
	lcd_command(LCD_DISP_ON_BLINK);
}

static void inc_raumtemp1(void)
{
	inc_val();
	display_raumtemp1();
}

static void dec_raumtemp1(void)
{
	dec_val();
	display_raumtemp1();
}

static void set_raumtemp1(void)
{
	eeprom_write_byte(&eraumtemp[0], currentval);
	show_raumniveau();
	
	currentval = eeprom_read_byte(&eraumtemp[1]);
	check_val();
	display_raumtemp2();
	lcd_command(LCD_DISP_ON_BLINK);
}


/*----------------------------------------------------
								FUNKTIONEN FR RAUMTEMP2
----------------------------------------------------*/
static void display_raumtemp2(void)
{
	lcd_gotoxy(14, 2);
	display_dezival(currentval, 0);
}

static void inc_raumtemp2(void)
{
	inc_val();
	display_raumtemp2();
}

static void dec_raumtemp2(void)
{
	dec_val();
	display_raumtemp2();
}

static void set_raumtemp2(void)
{
	eeprom_write_byte(&eraumtemp[1], currentval);
	show_raumniveau();
	
	currentval = eeprom_read_byte(&eniveau);
	minval = -99;
	maxval = 99;
	check_val();
	display_niveau();
	lcd_command(LCD_DISP_ON_BLINK);
}

/*----------------------------------------------------
								FUNKTIONEN FR NIVEAU
----------------------------------------------------*/
static void display_niveau(void)
{
	lcd_gotoxy(14, 3);
	display_dezival(currentval, 0);
}

static void inc_niveau(void)
{
	inc_val();
	display_niveau();
}

static void dec_niveau(void)
{
	dec_val();
	display_niveau();
}

static void set_niveau(void)
{
	eeprom_write_byte(&eniveau, currentval);
	go_raumtemp1();
}


/*----------------------------------------------------
								FUNKTIONEN FR BWPRIO
----------------------------------------------------*/

static void display_bwprio(void)
{
	lcd_gotoxy(2, 1);
	display_val(currentval, 0xDF);
}

static void go_bwprio(void)
{
	show_bwprio();
	currentval = eeprom_read_byte(&ebrauchwasserprio);
	minval = 25;
	maxval = 60;
	check_val();
	display_bwprio();
	lcd_command(LCD_DISP_ON_BLINK);	
}

static void inc_bwprio(void)
{
	inc_val();
	display_bwprio();
}

static void dec_bwprio(void)
{
	dec_val();
	display_bwprio();
}

static void set_bwprio(void)
{
	eeprom_write_byte(&ebrauchwasserprio, currentval);
	show_bwprio();
}


/*---------------------------------------------------------------------------
																MENU-DATA
	---------------------------------------------------------------------------*/

static const struct menu_item_t menu_items[] PROGMEM =
{
	//Info menues
	{MS_STATUS,				EVENT_PLUS,		show_brauchwasser,	MS_BRAUCHWASSER},
	{MS_STATUS,				EVENT_SECOND,	show_status,				MS_STATUS},
	{MS_STATUS,				EVENT_SET,		show_service,				MS_SERVICE},
	{MS_STATUS,				EVENT_BACK,		test_proc2,					MS_UNCHANGED},
	
	{MS_BRAUCHWASSER,	EVENT_MINUS,	show_status,				MS_STATUS},
	{MS_BRAUCHWASSER,	EVENT_SECOND, show_brauchwasser,	MS_BRAUCHWASSER},
	{MS_BRAUCHWASSER,	EVENT_SET,		show_service,				MS_SERVICE},
	{MS_BRAUCHWASSER,	EVENT_PLUS, 	show_heizung,				MS_HEIZUNG},
	
	{MS_HEIZUNG,			EVENT_MINUS,	show_brauchwasser,	MS_BRAUCHWASSER},
	{MS_HEIZUNG,			EVENT_SECOND,	show_heizung,				MS_HEIZUNG},
	{MS_HEIZUNG,			EVENT_SET,		show_service,				MS_SERVICE},
	{MS_HEIZUNG,			EVENT_PLUS,		show_holzkessel,		MS_HOLZKESSEL},
	
	{MS_HOLZKESSEL,		EVENT_MINUS,	show_heizung,				MS_HEIZUNG},
	{MS_HOLZKESSEL,		EVENT_SECOND,	show_holzkessel,		MS_HOLZKESSEL},
	{MS_HOLZKESSEL,		EVENT_SET,		show_service,				MS_SERVICE},
	{MS_HOLZKESSEL,		EVENT_PLUS,		show_oelkessel,			MS_OELKESSEL},
	{MS_HOLZKESSEL,		EVENT_BACK,		test_proc,					MS_UNCHANGED},
	
	{MS_OELKESSEL,		EVENT_MINUS,	show_holzkessel,		MS_HOLZKESSEL},
	{MS_OELKESSEL,		EVENT_SECOND,	show_oelkessel,			MS_OELKESSEL},
	{MS_OELKESSEL,		EVENT_SET,		show_service,				MS_SERVICE},
	
	//setup menues
	{MS_SERVICE,			EVENT_BACK,		show_status,				MS_STATUS},
	{MS_SERVICE,			EVENT_SET,		go_service,					MS_SETSERVICE},
	{MS_SERVICE,			EVENT_PLUS,		show_oelmode,				MS_OELMODE},

	{MS_OELMODE,			EVENT_MINUS,	show_service,				MS_SERVICE},
	{MS_OELMODE,			EVENT_BACK,		show_status,				MS_STATUS},
	{MS_OELMODE,			EVENT_SET,		go_oelmode,					MS_SETOELMODE},
	{MS_OELMODE,			EVENT_PLUS,		show_heizmode,			MS_HEIZMODE},

	{MS_HEIZMODE,			EVENT_MINUS,	show_oelmode,				MS_OELMODE},
	{MS_HEIZMODE,			EVENT_BACK,		show_status,				MS_STATUS},
	{MS_HEIZMODE,			EVENT_SET,		go_heizmode,				MS_SETHEIZMODE},
	{MS_HEIZMODE,			EVENT_PLUS,		show_heizkurve,			MS_HEIZKURVE},

	{MS_HEIZKURVE,		EVENT_MINUS,	show_heizmode,			MS_HEIZMODE},
	{MS_HEIZKURVE,		EVENT_BACK,		show_status,				MS_STATUS},
	{MS_HEIZKURVE,		EVENT_SET,		go_heizexponent,		MS_SETHEIZEXPONENT},
	{MS_HEIZKURVE,		EVENT_PLUS,		show_raumniveau,		MS_RAUMNIVEAU},
	
	{MS_RAUMNIVEAU,		EVENT_MINUS,	show_heizkurve,			MS_HEIZKURVE},
	{MS_RAUMNIVEAU,		EVENT_BACK,		show_status,				MS_STATUS},
	{MS_RAUMNIVEAU,		EVENT_SET,		go_raumtemp1,				MS_SETRAUMTEMP1},
	{MS_RAUMNIVEAU,		EVENT_PLUS,		show_heizgrenze,		MS_HEIZGRENZE},
	
	
	{MS_HEIZGRENZE,		EVENT_MINUS,	show_raumniveau,		MS_RAUMNIVEAU},
	{MS_HEIZGRENZE,		EVENT_BACK,		show_status,				MS_STATUS},
	{MS_HEIZGRENZE,		EVENT_SET,		test_proc2,					MS_UNCHANGED},
	{MS_HEIZGRENZE,		EVENT_PLUS,		show_bwprio,				MS_BWPRIO},
	
	{MS_BWPRIO,				EVENT_BACK,		show_status,				MS_STATUS},
	{MS_BWPRIO,				EVENT_MINUS,	show_heizgrenze,		MS_HEIZGRENZE},
	{MS_BWPRIO,				EVENT_SET,		go_bwprio,					MS_SETBWPRIO},
	
	
	{MS_SETSERVICE,		EVENT_BACK,		show_service,			MS_SERVICE},
	{MS_SETSERVICE,		EVENT_PLUS,		inc_service,			MS_SETSERVICE},
	{MS_SETSERVICE,		EVENT_MINUS,	dec_service,			MS_SETSERVICE},
	{MS_SETSERVICE,		EVENT_SET,		set_service,			MS_SETSERVICE},

	{MS_SETOELMODE,				EVENT_BACK,		show_oelmode,			MS_OELMODE},
	{MS_SETOELMODE,				EVENT_PLUS,		inc_oelmode,			MS_UNCHANGED},
	{MS_SETOELMODE,				EVENT_MINUS,	dec_oelmode,			MS_UNCHANGED},
	{MS_SETOELMODE,				EVENT_SET,		set_oelmode,			MS_SETOELSTART},

	{MS_SETOELSTART,			EVENT_BACK,		show_oelmode,			MS_OELMODE},
	{MS_SETOELSTART,			EVENT_PLUS,		inc_oelstart,			MS_UNCHANGED},
	{MS_SETOELSTART,			EVENT_MINUS,	dec_oelstart,			MS_UNCHANGED},
	{MS_SETOELSTART,			EVENT_SET,		set_oelstart,			MS_SETOELSTOP},

	{MS_SETOELSTOP,				EVENT_BACK,		show_oelmode,			MS_OELMODE},
	{MS_SETOELSTOP,				EVENT_PLUS,		inc_oelstop,			MS_UNCHANGED},
	{MS_SETOELSTOP,				EVENT_MINUS,	dec_oelstop,			MS_UNCHANGED},
	{MS_SETOELSTOP,				EVENT_SET,		set_oelstop,			MS_SETOELMODE},

	{MS_SETHEIZMODE,			EVENT_BACK,		show_heizmode,		MS_HEIZMODE},
	{MS_SETHEIZMODE,			EVENT_PLUS,		inc_heizmode,			MS_UNCHANGED},
	{MS_SETHEIZMODE,			EVENT_MINUS,	dec_heizmode,			MS_UNCHANGED},
	{MS_SETHEIZMODE,			EVENT_SET,		set_heizmode,			MS_SETHEIZSTART},

	{MS_SETHEIZSTART,			EVENT_BACK,		show_heizmode,		MS_HEIZMODE},
	{MS_SETHEIZSTART,			EVENT_PLUS, 	inc_heizstart,		MS_UNCHANGED},
	{MS_SETHEIZSTART,			EVENT_MINUS,	dec_heizstart,		MS_UNCHANGED},
	{MS_SETHEIZSTART,			EVENT_SET,		set_heizstart,		MS_SETHEIZSTOP},

	{MS_SETHEIZSTOP,			EVENT_BACK,		show_heizmode,		MS_HEIZMODE},
	{MS_SETHEIZSTOP,			EVENT_PLUS, 	inc_heizstop,			MS_UNCHANGED},
	{MS_SETHEIZSTOP,			EVENT_MINUS,	dec_heizstop,			MS_UNCHANGED},
	{MS_SETHEIZSTOP,			EVENT_SET,		set_heizstop,			MS_SETHEIZMODE},
	
	{MS_SETHEIZEXPONENT,	EVENT_BACK,		show_heizkurve,		MS_HEIZKURVE},
	{MS_SETHEIZEXPONENT,	EVENT_PLUS,		inc_heizexponent,	MS_UNCHANGED},
	{MS_SETHEIZEXPONENT,	EVENT_MINUS,	dec_heizexponent,	MS_UNCHANGED},
	{MS_SETHEIZEXPONENT,	EVENT_SET,		set_heizexponent,	MS_SETMAXVORLAUF},
	
	{MS_SETMAXVORLAUF,		EVENT_BACK,		show_heizkurve,		MS_HEIZKURVE},
	{MS_SETMAXVORLAUF,		EVENT_PLUS,		inc_maxvorlauf,		MS_UNCHANGED},
	{MS_SETMAXVORLAUF,		EVENT_MINUS,	dec_maxvorlauf,		MS_UNCHANGED},
	{MS_SETMAXVORLAUF,		EVENT_SET,		set_maxvorlauf,		MS_SETMINAUSSEN},
	
	{MS_SETMINAUSSEN,			EVENT_BACK,		show_heizkurve,		MS_HEIZKURVE},
	{MS_SETMINAUSSEN,			EVENT_PLUS,		inc_minaussen,		MS_UNCHANGED},
	{MS_SETMINAUSSEN,			EVENT_MINUS,	dec_minaussen,		MS_UNCHANGED},
	{MS_SETMINAUSSEN,			EVENT_SET,		set_minaussen,		MS_SETHEIZEXPONENT},
	
	{MS_SETRAUMTEMP1,			EVENT_BACK,		show_raumniveau,	MS_RAUMNIVEAU},
	{MS_SETRAUMTEMP1,			EVENT_PLUS,		inc_raumtemp1,		MS_UNCHANGED},
	{MS_SETRAUMTEMP1,			EVENT_MINUS,	dec_raumtemp1,		MS_UNCHANGED},
	{MS_SETRAUMTEMP1,			EVENT_SET,		set_raumtemp1,		MS_SETRAUMTEMP2},
	
	{MS_SETRAUMTEMP2,			EVENT_BACK,		show_raumniveau,	MS_RAUMNIVEAU},
	{MS_SETRAUMTEMP2,			EVENT_PLUS,		inc_raumtemp2,		MS_UNCHANGED},
	{MS_SETRAUMTEMP2,			EVENT_MINUS,	dec_raumtemp2,		MS_UNCHANGED},
	{MS_SETRAUMTEMP2,			EVENT_SET,		set_raumtemp2,		MS_SETNIVEAU},
	
	{MS_SETNIVEAU,				EVENT_BACK,		show_raumniveau,	MS_RAUMNIVEAU},
	{MS_SETNIVEAU,				EVENT_PLUS,		inc_niveau,				MS_UNCHANGED},
	{MS_SETNIVEAU,				EVENT_MINUS,	dec_niveau,				MS_UNCHANGED},
	{MS_SETNIVEAU,				EVENT_SET,		set_niveau,				MS_SETRAUMTEMP1},
	
	{MS_SETBWPRIO,				EVENT_BACK,		show_bwprio,			MS_BWPRIO},
	{MS_SETBWPRIO,				EVENT_PLUS,		inc_bwprio,				MS_UNCHANGED},
	{MS_SETBWPRIO,				EVENT_MINUS,	dec_bwprio,				MS_UNCHANGED},
	{MS_SETBWPRIO,				EVENT_SET,		set_bwprio,				MS_BWPRIO},
	
	//{MS_HEIZUNG,		EVENT_BACK,		test_proc2,				MS_UNCHANGED},
	{MS_HOLZKESSEL,		EVENT_BACK,		test_proc,				MS_UNCHANGED},
	
	
	{0,					0,				0,						0}
};


void menu_event(unsigned char event)
{
	struct menu_item_t menu_item;
	unsigned char i=0;
	
	do
	{
		memcpy_P(&menu_item, &menu_items[i++], sizeof(struct menu_item_t));
	}

	while (menu_item.state && ((menu_item.event != event) || (menu_item.state != menu_state)));
	
	if (menu_item.state)
	{
		if (menu_item.nextstate != MS_UNCHANGED)
		{
			lcd_clrscr();
			menu_state=menu_item.nextstate;
		}
		menu_item.proc();
	}
}
