#include <avr/io.h>
#include <avr/eeprom.h>
#include "oel.h"
#include "relais.h"
#include "holz.h"
#include "fuehler.h"
#include "heizung.h"
#include "brauchwasser.h"
#include "menu.h"
#include "dcf77.h"

unsigned char oel_status = OEL_AUS;

unsigned char eoel_timer[2] __attribute__ ((section (".eeprom"))) = {4, 32};
unsigned char eoel_mode __attribute__ ((section (".eeprom"))) = 2;

unsigned char oel_laststart_h;
unsigned char oel_laststart_m;


static unsigned char oel_timer(void)
{
	
	unsigned char thistime = dcf77.hour * 4 + dcf77.minute / 15;

	if (eeprom_read_byte(&eoel_timer[0]) <= eeprom_read_byte(&eoel_timer[1]))
		if ((thistime >= eeprom_read_byte(&eoel_timer[0])) && (thistime < eeprom_read_byte(&eoel_timer[1])))
			return 1;
		else
			return 0;
	else
		if ((thistime >= eeprom_read_byte(&eoel_timer[0])) || (thistime < eeprom_read_byte(&eoel_timer[1])))
			return 1;
		else
			return 0;
}

void oel_trigger(void)
{
	signed char oel_req = 0;
	
	unsigned char om = eeprom_read_byte(&eoel_mode);

	if ((oel_status == OEL_AUS) || (oel_status == OEL_NACHLAUF))
	{
		if (holz_status == HOLZ_AUS)
		{//schauen ob Oel angeschalted werden muss
			if ((heizung_status == HEIZUNG_NOENERGY) && ((om == OEL_MODE_HEIZUNG) || (om == OEL_MODE_HEIZUNGBW)))
				oel_req = 1;

			if ((brauchwasser_status == BRAUCHWASSER_NOENERGY) && ((om == OEL_MODE_BRAUCHWASSER) || (om == OEL_MODE_HEIZUNGBW)))
				oel_req = 1;

			if (oel_timer() && oel_req)
			{
				RELAIS_OEL_ON;
				oel_status = OEL_AUFHEIZEN;
				oel_laststart_h = dcf77.hour;
				oel_laststart_m = dcf77.minute;
			}
		}
	}
	else
	{
		if ((brauchwasser_status == BRAUCHWASSER_OK) || ((om != OEL_MODE_HEIZUNG) && (om != OEL_MODE_HEIZUNGBW)))
			oel_req -= 1;

		if ((heizung_status == HEIZUNG_AUS) || ((om != OEL_MODE_BRAUCHWASSER) && (om != OEL_MODE_HEIZUNGBW)))
			oel_req -= 1;

		if (holz_status != HOLZ_AUS)
			oel_req = -2;

		if (!oel_timer() || (oel_req == -2))
		{
			RELAIS_OEL_OFF;
			if (oel_status == OEL_EIN)
				oel_status = OEL_NACHLAUF;
			else
				oel_status = OEL_AUS;
		}
	}

	/*if (holz_status == HOLZ_AUS)
	{
		if (oel_status == OEL_AUS)
		{//guggn ob lbrenner aktiviert werden muss
			if ((heizung_status == HEIZUNG_NOENERGY) && (oel_flags & (1<<OELFLAG_HEIZUNG)))
				oel_req = 1;
				
			if ((brauchwasser_status == BRAUCHWASSER_NOENERGY) && (oel_flags & (1<<OELFLAG_BRAUCHWASSER)))
				oel_req = 1;
			
			if (oel_req)
				oel_status = OEL_AUFHEIZEN;
		}
		else
		{ //guggn ob lbrenner immer noch gebraucht wird
			
			if ((heizung_status == HEIZUNG_AUS) || (!(oel_flags & (1<<OELFLAG_HEIZUNG))))
				oel_req += 1;
				
			if ((brauchwasser_status == BRAUCHWASSER_OK) || (!(oel_flags & (1<<OELFLAG_BRAUCHWASSER))))
				oel_req += 1;
				
			if (oel_req == 2)
				oel_status = OEL_AUS;
		}
	}
	else
		oel_status = OEL_AUS;*/
	
	switch (oel_status)
	{
		case OEL_AUFHEIZEN:
			if (get_fuehler(NTC_OEL_VORLAUF) > (OEL_MIN_TEMP + 10))
			{
				oel_status = OEL_EIN;
				RELAIS_MIXER_OEL;
			}
			break;
		
		case OEL_EIN:
			if (get_fuehler(NTC_OEL_VORLAUF) < (OEL_MIN_TEMP - 10))
			{
				oel_status = OEL_AUFHEIZEN;
				RELAIS_MIXER_HOLZ;
			}
			break;
			
		case OEL_NACHLAUF:
			if ((heizung_status != HEIZUNG_EIN) && (brauchwasser_status != BRAUCHWASSER_AUFHEIZEN))
			{
				oel_status = OEL_AUS;
				RELAIS_MIXER_HOLZ;
			}
			break;
				
		case OEL_AUS:
			RELAIS_MIXER_HOLZ;
			break;

			
	}
}
