#include <avr/io.h>
#include <avr/eeprom.h>
#include "fuehler.h"
#include "holz.h"
#include "oel.h"
#include "source.h"
#include "dcf77.h"
#include "heizung.h"
#include "brauchwasser.h"

unsigned char get_sourcetype(void)
{
	switch (oel_status)
	{
		case OEL_AUFHEIZEN:
			return SOURCE_PUFFER;
		
		case OEL_EIN:
		case OEL_NACHLAUF:
			return SOURCE_OEL;
	}
	
	//oel ist aus...
	
	if (holz_status == HOLZ_BRENNT)
		return SOURCE_HOLZKESSEL;
	
	return SOURCE_PUFFER;
}

signed short get_sourcetemp(void)
{
	switch (get_sourcetype())
	{
		case SOURCE_OEL:
			return get_fuehler(NTC_OEL_VORLAUF);
			
		case SOURCE_HOLZKESSEL:
			return get_fuehler(NTC_HOLZ_VORLAUF);
	}
	return get_fuehler(NTC_PUFFER_OBEN);
}

void source_trigger(void)
{
	unsigned short bwprio;
	
	holz_trigger();
	
	bwprio = eeprom_read_byte(&ebrauchwasserprio) * 10;
	
	brauchwasser_trigger();

	if (
				(
					(brauchwasser_status == BRAUCHWASSER_AUFHEIZEN) ||
					(
						(brauchwasser_status == BRAUCHWASSER_NOENERGY) &&
						(get_sourcetype() != SOURCE_PUFFER)
					)
				) && 
			
				(get_fuehler(NTC_BOILER_OBEN) < bwprio - BRAUCHWASSERHYST)
			)
	{
		if (heizung_status == HEIZUNG_EIN)
			heizung_status = HEIZUNG_BWVORRANG;
	}
	else
		if (heizung_status == HEIZUNG_BWVORRANG)
			heizung_status = HEIZUNG_EIN;

	
	heizung_statetrigger();

	oel_trigger();
}
