#include <avr/io.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "holz.h"
#include "brauchwasser.h"
#include "oel.h"
#include "heizung.h"
#include "fuehler.h"
#include "dcf77.h"
#include <string.h>
#include <stdlib.h>

static char txbuf[196];
static unsigned short txpointer = 0;
static unsigned char txbuf_len = 0;

SIGNAL (SIG_UART_DATA)
{
	if (txbuf_len--)
		UDR=txbuf[txpointer++];
	else
		UCSRB&=~(1<<UDRIE);
}

static void add_value(signed short value, unsigned char format)
{
	unsigned char i;
	switch (format)
	{
		case 0: //x,x
			itoa(value / 10, &txbuf[txbuf_len], 10);
			i = strlen(&txbuf[txbuf_len]);
			txbuf_len += i;
			
			txbuf[txbuf_len++] = '.';
			utoa(abs(value) % 10, &txbuf[txbuf_len++], 10);
			break;
		
		case 1:
			utoa(value, &txbuf[txbuf_len], 10);
			txbuf_len += strlen(&txbuf[txbuf_len]);
			break;
	}
	
	txbuf[txbuf_len++] = ';';
}

static void add_datetime_atom(unsigned char value)
{
	if (value < 10)
		txbuf[txbuf_len++] = '0';
		
	utoa(value, &txbuf[txbuf_len], 10);
	txbuf_len += strlen(&txbuf[txbuf_len]);
}

/* INFOBROADCAST DATENSATZ AUFBAU

	<ESC><0x10><ASCII Text><ESC><0x11>
	
	Beschreibung	Format
	-------------------------------
	DATETIME			02.03.05 08:13:24
	DayOfWeek			3
	Solar					7.8
	BoilerUnten		20.2
	BoilerOben		40.0
	PufferUnten		53.6
	PufferOben		59.1
	HolzVorlauf		74.4
	HolzRcklauf	67.8
	HeizungVorl		32.4
	HeizungRckl	29.0
	lVorlauf			19.1
	lrcklauf		18.9
	Aussen				5.0
	HeizSoll			32.1
	RELAIS				196
	BrauchwST			3
	HeizungST			2
	HolzST				1
	lST					0
	
	
	
	

*/


void uart_trigger(void)
{
	unsigned char i;
	unsigned short relais;
	txbuf[0] = 27;
	txbuf[1] = 0x10;
	txbuf_len = 2;
	
	add_datetime_atom(dcf77.day);
	txbuf[txbuf_len++] = '.';
	add_datetime_atom(dcf77.month);
	txbuf[txbuf_len++] = '.';
	add_datetime_atom(dcf77.year);
	txbuf[txbuf_len++] = ' ';
	
	add_datetime_atom(dcf77.hour);
	txbuf[txbuf_len++] = ':';
	add_datetime_atom(dcf77.minute);
	txbuf[txbuf_len++] = ':';
	add_datetime_atom(dcf77.second);
	txbuf[txbuf_len++] = ';';
	
	add_value(dcf77.dayofweek, 1);
		
	for(i=0;i<=11;i++)
		add_value(get_fuehler(i), 0);
	

	add_value(get_heizung_soll(), 0);
	relais=PORTA | ((PORTC & 0xC0) << 2);
	add_value(relais, 1);
	add_value(brauchwasser_status, 1);
	add_value(heizung_status, 1);
	add_value(holz_status, 1);
	add_value(oel_status, 1);
	add_value(get_mixer_percent(), 1);
	
	txbuf[txbuf_len++] = 27;
	txbuf[txbuf_len++] = 0x11;
	
	txpointer=0;
	UCSRB |= 1<<UDRIE;
}
